@@ -54,6 +54,8 @@ async def _load_design(
     preload: bool = False,
     no_clone: bool = False,
     scale: float | None = None,
+    kludge_factor: float | None = None,
+    epoch: float | None = None,
     epoch_delay: float = 0.0,
     get_paths: bool = True,
 ):
@@ -115,6 +117,8 @@ async def _load_design(
             clip_scale: float = config["configuration"]["clip_scale"]
             SCALE_KLUDGE: float = config["configuration"]["scale_kludge_factor"]
 
+            guider_scale: float | None = None
+
             # Query the guider for the historical scale from the previous exposure.
             command.debug("Getting guider scale.")
             get_scale_cmd = await command.send_command(
@@ -123,7 +127,13 @@ async def _load_design(
             )
             if get_scale_cmd.status.did_fail:
                 command.warning("Failed getting scale from guider.")
+            else:
+                guider_scale = float(get_scale_cmd.replies.get("scale_median")[0])
+                if guider_scale < 0:
+                    command.warning("Invalid guider scale.")
+                    guider_scale = None
 
+            if guider_scale is None:
                 # Try using the scale-temperature relationship instead.
                 try:
                     if not isinstance(command.actor.fps.ieb, IEB):
@@ -131,14 +141,14 @@ async def _load_design(
 
                     temperature = (await command.actor.fps.ieb.read_device("T3"))[0]
                     if not isinstance(temperature, float) or temperature < -100:
-                        raise ValueError("invalid ambient temperature")
+                        raise ValueError("Invalid ambient temperature")
 
                     coeffs = config["configuration"]["scale_temperature_coeffs"]
-                    scale = numpy.polyval(coeffs, temperature)
+                    guider_scale = numpy.polyval(coeffs, temperature)  # type:ignore
 
                     command.debug(
                         "Using focal scale factor derived from ambient "
-                        f"temperature ({temperature:.2f} C): {scale}"
+                        f"temperature ({temperature:.2f} C): {guider_scale:.6f}"
                     )
                 except Exception as err:
                     command.warning(
@@ -147,33 +157,38 @@ async def _load_design(
                     )
 
             else:
-                guider_scale = float(get_scale_cmd.replies.get("scale_median")[0])
-                if guider_scale < 0:
+                if (abs(guider_scale) - 1) * 1e6 > clip_scale:
+                    guider_scale = numpy.clip(
+                        guider_scale,
+                        1 - clip_scale / 1e6,
+                        1 + clip_scale / 1e6,
+                    )
                     command.warning(
-                        "Invalid guider scale. No scale correction will be applied."
+                        "Unexpectedly large guider scale. "
+                        f"Clipping to {guider_scale:.6f}."
                     )
-                else:
-                    if (abs(guider_scale) - 1) * 1e6 > clip_scale:
-                        guider_scale = numpy.clip(
-                            guider_scale,
-                            1 - clip_scale / 1e6,
-                            1 + clip_scale / 1e6,
-                        )
-                        command.warning(
-                            "Unexpectedly large guider scale. "
-                            f"Clipping to {guider_scale}."
-                        )
 
-                    scale = FOCAL_SCALE * guider_scale * SCALE_KLUDGE
-                    command.debug(
-                        "Text correcting focal plane scale with guider scale "
-                        f"{guider_scale}. Effective focal plane scale is {scale}."
-                    )
+            if guider_scale is not None:
+
+                kludge_factor = kludge_factor or SCALE_KLUDGE
+
+                command.debug(f"Scale kludge factor: {kludge_factor:.6f}")
+
+                scale = FOCAL_SCALE * guider_scale * kludge_factor
+
+                command.debug(
+                    "Text correcting focal plane scale with guider scale "
+                    f"{guider_scale:.6f}. Effective focal plane scale is {scale:.6f}."
+                )
 
         try:
             # Define the epoch for the configuration.
             command.debug(text=f"Epoch delay: {round(epoch_delay, 1)} seconds.")
-            epoch = Time.now().jd + epoch_delay / 86400.0
+
+            if epoch is None:
+                epoch = float(Time.now().jd)
+            epoch += epoch_delay / 86400.0
+
             design = await Design.create_async(design_id, epoch=epoch, scale=scale)
         except Exception as err:
             command.error(error=f"Failed retrieving design: {err}")
@@ -211,6 +226,11 @@ def configuration():
     default=True,
     help="Generates and stores the to and from destination paths.",
 )
+@click.option(
+    "--epoch",
+    type=float,
+    help="Epoch at which to create the configuration, as a Julian Day.",
+)
 @click.option(
     "--epoch-delay",
     type=float,
@@ -253,6 +273,12 @@ def configuration():
     type=float,
     help="Focal plane scale factor. If not passes, uses coordio default.",
 )
+@click.option(
+    "--kludge-factor",
+    type=float,
+    default=None,
+    help="Modify the default scale kludge factor.",
+)
 @click.option(
     "--no-clone",
     is_flag=True,
@@ -269,12 +295,14 @@ async def load(
     from_positions: bool = False,
     from_preloaded: bool = False,
     generate_paths: bool = False,
+    epoch: float | None = None,
     epoch_delay: float = 0.0,
     ingest: bool = False,
     write_summary: bool = False,
     execute: bool = False,
     reissue: bool = False,
     scale: float | None = None,
+    kludge_factor: float | None = None,
     no_clone: bool = False,
 ):
     """Creates and ingests a configuration from a design in the database."""
@@ -311,9 +339,11 @@ async def load(
             command,
             fps,
             design_id=designid,
+            kludge_factor=kludge_factor,
             preload=False,
             no_clone=no_clone,
             scale=scale,
+            epoch=epoch,
             epoch_delay=epoch_delay,
             get_paths=False,
         )
@@ -338,6 +368,7 @@ async def load(
         not fps.configuration.is_cloned
         and generate_paths
         and fps.configuration.to_destination is None
+        and not from_positions
     ):
         try:
             command.info("Calculating trajectories.")
@@ -370,7 +401,7 @@ async def load(
     snapshot = await fps.configuration.save_snapshot()
     command.info(configuration_snapshot=snapshot)
 
-    if execute:
+    if execute and not from_positions:
         cmd = await command.send_command("jaeger", "configuration execute")
         if cmd.status.did_fail:
             if cmd.status.is_done:
@@ -400,6 +431,28 @@ async def load(
     return command.finish(f"Configuration {fps.configuration.configuration_id} loaded.")
 
 
+@configuration.command()
+async def reload(command: Command[JaegerActor], fps: FPS):
+    """Reloads the current configuration.
+
+    This is equivalent to using jaeger configuration load --no-clone DESIGNID
+    where DESIGNID is the currently loaded design.
+
+    """
+
+    if (
+        fps.configuration is None
+        or fps.configuration.design_id is None
+        or fps.configuration.design_id == -999
+    ):
+        return command.fail("There is no configuration currently loaded.")
+
+    design_id = fps.configuration.design_id
+    await command.child_command(f"configuration load --no-clone {design_id}")
+
+    return command.finish()
+
+
 def _output_configuration_loaded(command: Command[JaegerActor], fps: FPS):
     """Outputs the loaded configuration."""
 
@@ -442,6 +495,11 @@ async def clone(command: Command[JaegerActor], fps: FPS):
 
 
 @configuration.command(cancellable=True)
+@click.option(
+    "--epoch",
+    type=float,
+    help="Epoch at which to create the configuration, as a Julian Day.",
+)
 @click.option(
     "--epoch-delay",
     type=float,
@@ -453,6 +511,12 @@ async def clone(command: Command[JaegerActor], fps: FPS):
     type=float,
     help="Focal plane scale factor. If not passes, uses coordio default.",
 )
+@click.option(
+    "--kludge-factor",
+    type=float,
+    default=None,
+    help="Modify the default scale kludge factor.",
+)
 @click.option(
     "--no-clone",
     is_flag=True,
@@ -474,8 +538,10 @@ async def preload(
     command: JaegerCommandType,
     fps: FPS,
     designid: int | None = None,
+    epoch: float | None = None,
     epoch_delay: float = 0.0,
     scale: float | None = None,
+    kludge_factor: float | None = None,
     no_clone: bool = False,
     make_active: bool = True,
     clear: bool = False,
@@ -501,6 +567,8 @@ async def preload(
         preload=True,
         no_clone=no_clone,
         scale=scale,
+        kludge_factor=kludge_factor,
+        epoch=epoch,
         epoch_delay=epoch_delay,
     )
 
@@ -651,23 +719,46 @@ async def reverse(command: Command[JaegerActor], fps: FPS, explode: bool = False
 
 @configuration.command()
 @click.argument("RADIUS", type=float)
-async def dither(command: Command[JaegerActor], fps: FPS, radius: float):
+@click.option(
+    "--epoch",
+    type=str,
+    default="inherit",
+    help="Epoch to use for the dithered configuration. With 'inherit', the parent "
+    "configuration epoch is used; 'now' uses the current time. Otherwise provide a JD.",
+)
+async def dither(
+    command: Command[JaegerActor],
+    fps: FPS,
+    radius: float,
+    epoch: str = "inherit",
+):
     """Dither a loaded configuration."""
 
     if fps.configuration is None:
         return command.fail("A configuration must first be loaded.")
 
-    if isinstance(fps.configuration, DitheredConfiguration):
+    is_dithered = isinstance(fps.configuration, DitheredConfiguration)
+    if is_dithered and fps.configuration.parent_configuration is not None:
         parent_configuration = fps.configuration.parent_configuration
     else:
         parent_configuration = fps.configuration
 
-    command.info(
-        "Creating dithered configuration from "
-        f"{parent_configuration.configuration_id}."
-    )
+    configuration_id = parent_configuration.configuration_id
+    command.info(f"Creating dithered configuration from {configuration_id}")
 
-    fps.configuration = DitheredConfiguration(parent_configuration, radius)
+    epoch = epoch.lower()
+    if epoch == "inherit":
+        dither_epoch = None
+    elif epoch == "now":
+        dither_epoch = Time.now().jd
+    else:
+        dither_epoch = float(epoch)
+
+    fps.configuration = DitheredConfiguration(
+        parent_configuration,
+        radius,
+        epoch=dither_epoch,
+    )
     await fps.configuration.get_paths()
 
     fps.configuration.write_to_database()
@@ -682,10 +773,8 @@ async def dither(command: Command[JaegerActor], fps: FPS, radius: float):
 
     fps.configuration.executed = True
 
-    command.finish(
-        f"Dithered configuration {fps.configuration.configuration_id} "
-        "loaded and executed."
-    )
+    configuration_id = fps.configuration.configuration_id
+    command.finish(f"Dithered configuration {configuration_id} loaded and executed.")
 
 
 @configuration.command()
@@ -755,6 +844,12 @@ async def slew(
     help="Send the trajectory to the FPS.",
     default=True,
 )
+@click.option(
+    "--max-retries",
+    type=int,
+    default=10,
+    help="Home many retries to allow due to deadlocks.",
+)
 async def random(
     command: Command[JaegerActor],
     fps: FPS,
@@ -763,6 +858,7 @@ async def random(
     uniform: str | None = None,
     collision_buffer: float | None = None,
     send_trajectory: bool = True,
+    max_retries: int = 10,
 ):
     """Executes a random, valid configuration."""
 
@@ -773,11 +869,14 @@ async def random(
 
     # Check that all positioners are folded.
     await fps.update_position()
-    positions = fps.get_positions()
+    positions = fps.get_positions(ignore_disabled=True)
 
     if len(positions) == 0:
         return command.fail("No positioners connected")
 
+    alphaL: float
+    betaL: float
+
     alphaL, betaL = config["kaiju"]["lattice_position"]
     if not numpy.allclose(positions[:, 1:] - [alphaL, betaL], 0, atol=1):
         return command.fail(error="Not all the positioners are folded.")
@@ -793,17 +892,21 @@ async def random(
     else:
         uniform_unpack = None
 
-    configuration = await create_random_configuration(
-        fps,
-        seed=seed,
-        uniform=uniform_unpack,
-        safe=not danger,
-        collision_buffer=collision_buffer,
-    )
-
     try:
+        configuration = await create_random_configuration(
+            fps,
+            seed=seed,
+            uniform=uniform_unpack,
+            safe=not danger,
+            collision_buffer=collision_buffer,
+            max_retries=max_retries,
+        )
+
         command.info("Getting trajectory.")
-        trajectory = await configuration.get_paths(decollide=False)
+        trajectory = await configuration.get_paths(
+            decollide=False,
+            collision_buffer=collision_buffer,
+        )
     except JaegerError as err:
         return command.fail(error=f"jaeger random failed: {err}")
 
@@ -821,4 +924,6 @@ async def random(
     except TrajectoryError as err:
         return command.fail(error=f"Trajectory failed with error: {err}")
 
+    configuration.executed = True
+
     command.finish()
