@@ -23,40 +23,65 @@ from astropy.table import Table
 from clu.command import Command
 from clu.legacy.tron import TronConnection
 from coordio.defaults import calibration
-from coordio.transforms import FVCTransformAPO
 
 from jaeger import config, log
 from jaeger.exceptions import FVCError, JaegerUserWarning, TrajectoryError
 from jaeger.fps import FPS
 from jaeger.ieb import IEB
 from jaeger.kaiju import get_path_pair_in_executor, get_robot_grid
-from jaeger.target.tools import wok_to_positioner
+from jaeger.plotting import plot_fvc_distances
+from jaeger.target import Configuration, Design
+from jaeger.target.tools import read_confSummary, wok_to_positioner
 from jaeger.utils import run_in_executor
 
 
 if TYPE_CHECKING:
+    from coordio.transforms import FVCTransformAPO, FVCTransformLCO
+
     from jaeger.actor import JaegerActor
+    from jaeger.target.configuration import BaseConfiguration
 
 
 __all__ = ["FVC"]
 
 
 FVC_CONFIG = config["fvc"]
+DEFAULT_CENTROID_METHOD = "zbplus"
+
+
+def get_transform(observatory: str):
+    """Returns the correct coordio FVC transform class for the observatory."""
+
+    if observatory.upper() == "APO":
+        from coordio.transforms import FVCTransformAPO
+
+        return FVCTransformAPO
+
+    elif observatory.upper() == "LCO":
+        from coordio.transforms import FVCTransformLCO
+
+        return FVCTransformLCO
+
+    else:
+        raise ValueError(f"Invalid observatory {observatory}.")
 
 
 class FVC:
     """Focal View Camera class."""
 
+    configuration: Optional[BaseConfiguration]
     fibre_data: Optional[pandas.DataFrame]
     measurements: Optional[pandas.DataFrame]
     centroids: Optional[pandas.DataFrame]
     offsets: Optional[pandas.DataFrame]
+    fvc_transform: Optional[FVCTransformAPO | FVCTransformLCO]
 
     image_path: Optional[str]
     proc_image_path: Optional[str]
     raw_hdu: Optional[fits.ImageHDU]
     proc_hdu: Optional[fits.ImageHDU]
 
+    centroid_method: str | None
     fitrms: float
     k: float
 
@@ -79,7 +104,8 @@ class FVC:
     def reset(self):
         """Resets the instance."""
 
-        self.fvc_transform: FVCTransformAPO | None = None
+        self.configuration = None
+        self.fvc_transform: FVCTransformAPO | FVCTransformLCO | None = None
         self.fibre_data = None
         self.centroids = None
         self.offsets = None
@@ -93,6 +119,7 @@ class FVC:
         self.fitrms = -9.99
         self.perc_90 = -9.99
         self.fvc_percent_reached = -9.99
+        self.centroid_method = None
 
     def set_command(self, command: Command[JaegerActor]):
         """Sets the command."""
@@ -191,9 +218,10 @@ class FVC:
         self,
         path: pathlib.Path | str,
         positioner_coords: dict,
+        configuration: Optional[BaseConfiguration] = None,
         fibre_data: Optional[pandas.DataFrame] = None,
         fibre_type: str = "Metrology",
-        use_winpos: bool = True,
+        centroid_method: str | None = None,
         use_new_invkin: bool = True,
         plot: bool | str = False,
         outdir: str | None = None,
@@ -207,6 +235,9 @@ class FVC:
         positioner_coords
             A dictionary of positioner ID to ``(alpha, beta)`` with the current
             positions of the robots.
+        configuration
+            A configuration object to use for processing. If `None`, defaults to the
+            current `.FPS` loaded configuration.
         fibre_data
             A Pandas data frame with the expected coordinates of the targets. It
             is expected the data frame will have columns ``positioner_id``,
@@ -217,8 +248,9 @@ class FVC:
         fibre_type
             The ``fibre_type`` rows in ``fibre_data`` to use. Defaults to
             ``fibre_type='Metrology'``.
-        use_winpos
-            Whether to use windowed position for centroid extraction.
+        centroid_method
+            The centroid method to use, one of ``"nudge"``, ``"sep"``, ``"winpos"``,
+            or ``"simple"``. Defaults to ``"nudge"``.
         use_new_invkin
             Use new inverse kinnematic to calculate alpha/beta.
         plot
@@ -239,17 +271,25 @@ class FVC:
         # Reset the instance
         self.reset()
 
+        centroid_method = centroid_method or DEFAULT_CENTROID_METHOD
+
         path = str(path)
         if not os.path.exists(path):
             raise FVCError(f"FVC image {path} does not exist.")
 
+        self.configuration = configuration or self.fps.configuration
+
         if fibre_data is None:
-            if self.fps is None or self.fps.configuration is None:
+            if configuration is None:
                 raise FVCError("No fibre data and no configuration has been loaded.")
-            fibre_data = self.fps.configuration.assignment_data.fibre_table
+            fibre_data = configuration.assignment_data.fibre_table
 
         fdata = fibre_data.copy().reset_index().set_index("positioner_id")
 
+        # Set fibre_data initially so that if FVCTransform fails we have something.
+        self.fibre_data = fdata.copy().reset_index()
+        self.fibre_data.set_index(["hole_id", "fibre_type"], inplace=True)
+
         self.log(f"Processing raw image {path}")
 
         dirname, base = os.path.split(path)
@@ -259,6 +299,8 @@ class FVC:
 
         if plot is True:
             plot_path_root = proc_path_root
+        elif plot is False or plot is None:
+            plot_path_root = None
         elif isinstance(plot, str):
             plot_path_root = os.path.join(plot, "proc-" + base[0 : base.find(".fit")])
         else:
@@ -268,6 +310,7 @@ class FVC:
 
         self.image_path = path
         self.raw_hdu = hdus[1].copy()
+        self.proc_hdu = hdus[1]
 
         # Invert columns
         hdus[1].data = hdus[1].data[:, ::-1]
@@ -277,7 +320,7 @@ class FVC:
 
         # Get the rotator angle so that we can derotate the centroids to the
         # x/ywok-aligned configuration.
-        rotpos = hdus[1].header.get("IPA", None)
+        rotpos = hdus[1].header.get("IPA", 135.4)
         if rotpos is None:
             raise FVCError("IPA keyword not found in the header.")
         rotpos = float(rotpos) % 360.0
@@ -289,7 +332,10 @@ class FVC:
         )
         positioner_df.index.set_names(["positionerID"], inplace=True)
 
-        fvc_transform = FVCTransformAPO(
+        FVCTransform = get_transform(self.fps.observatory)
+        self.log(f"Using FVC transform class {FVCTransform!r}.")
+
+        fvc_transform = FVCTransform(
             image_data,
             positioner_df,
             rotpos,
@@ -297,7 +343,11 @@ class FVC:
         )
 
         self.centroids = fvc_transform.extractCentroids()
-        fvc_transform.fit(useWinpos=use_winpos, newInvKin=use_new_invkin)
+        fvc_transform.fit(centType=centroid_method, newInvKin=use_new_invkin)
+        self.centroid_method = centroid_method
+
+        if self.command:
+            self.command.info(fvc_centroid_method=self.centroid_method)
 
         assert fvc_transform.positionerTableMeas is not None
 
@@ -376,7 +426,6 @@ class FVC:
         fdata.set_index(["hole_id", "fibre_type"], inplace=True)
 
         self.fibre_data = fdata
-        self.proc_hdu = hdus[1]
         self.fvc_transform = fvc_transform
 
         self.log(f"Finished processing {path}", level=logging.DEBUG)
@@ -620,18 +669,13 @@ class FVC:
             image_path = pathlib.Path(self.image_path)
             new_filename = image_path.with_name("proc-" + image_path.name)
 
-        if (
-            self.fibre_data is None
-            or self.centroids is None
-            or self.raw_hdu is None
-            or self.proc_hdu is None
-        ):
+        if self.fibre_data is None or self.raw_hdu is None or self.proc_hdu is None:
             raise FVCError("Need to run process_fvc_image before writing the image.")
 
         proc_hdus = fits.HDUList([fits.PrimaryHDU(), self.proc_hdu])
 
-        if self.fps and self.fps.configuration:
-            proc_hdus[1].header["CONFIGID"] = self.fps.configuration.configuration_id
+        if self.configuration:
+            proc_hdus[1].header["CONFIGID"] = self.configuration.configuration_id
         else:
             proc_hdus[1].header["CONFIGID"] = -999.0
 
@@ -647,6 +691,16 @@ class FVC:
             ("FIDUCIALCOORDS", fiducialCoords.reset_index()),
         ]
 
+        if self.fvc_transform is not None:
+            if self.fvc_transform.positionerTableMeas is not None:
+                pos_table_meas = self.fvc_transform.positionerTableMeas
+                pos_table_meas = pos_table_meas.drop(columns=["index"])
+                dfs.append(("POSITIONERTABLEMEAS", pos_table_meas))
+            if self.fvc_transform.fiducialCoordsMeas is not None:
+                fid_coords_meas = self.fvc_transform.fiducialCoordsMeas
+                fid_coords_meas = fid_coords_meas.drop(columns=["index"])
+                dfs.append(("FIDUCIALCOORDSMEAS", fid_coords_meas))
+
         for name, df in dfs:
             rec = Table.from_pandas(df).as_array()
             table = fits.BinTableHDU(rec, name=name)
@@ -677,7 +731,10 @@ class FVC:
 
         # Add header keywords from coordio.FVCTransfromAPO.
         if self.fvc_transform:
-            proc_hdus[1].header.extend(self.fvc_transform.getMetadata())
+            try:
+                proc_hdus[1].header.extend(self.fvc_transform.getMetadata())
+            except Exception as err:
+                self.log(f"Cannot get FVCTransform metadata: {err}", logging.WARNING)
 
         posangles = None
         if self.fps:
@@ -696,8 +753,8 @@ class FVC:
             current_positions["startAlpha"] = numpy.nan
             current_positions["startBeta"] = numpy.nan
 
-            if self.fps.configuration:
-                robot_grid = self.fps.configuration.robot_grid
+            if self.configuration:
+                robot_grid = self.configuration.robot_grid
 
                 _cmd_alpha = []
                 _cmd_beta = []
@@ -720,7 +777,10 @@ class FVC:
             posangles = Table.from_pandas(current_positions).as_array()
         proc_hdus.append(fits.BinTableHDU(posangles, name="POSANGLES"))
 
-        rec = Table.from_pandas(self.centroids).as_array()
+        if self.centroids is not None:
+            rec = Table.from_pandas(self.centroids).as_array()
+        else:
+            rec = None
         proc_hdus.append(fits.BinTableHDU(rec, name="CENTROIDS"))
 
         offsets = None
@@ -809,10 +869,15 @@ class FVC:
         self.correction_applied = True
         self.log("Correction applied.")
 
-    async def write_summary_F(self):
+    async def write_summary_F(
+        self,
+        path: str | pathlib.Path | None = None,
+        plot: bool = True,
+        extra_headers: dict = {},
+    ):
         """Updates data with the last measured positions and write confSummaryF."""
 
-        if self.fps is None or self.fps.configuration is None:
+        if self.configuration is None:
             raise FVCError("write_summary_F can only be called with a configuration.")
 
         if self.fibre_data is None:
@@ -841,7 +906,7 @@ class FVC:
             if not numpy.isnan(alpha):
                 fdata.loc[idx[pid, :], ["alpha", "beta"]] = (alpha, beta)
 
-        configuration_copy = self.fps.configuration.copy()
+        configuration_copy = self.configuration.copy()
         configuration_copy.assignment_data.fibre_table = fdata.copy()
 
         for pid in measured.index.get_level_values(0).tolist():
@@ -855,13 +920,141 @@ class FVC:
                     update=True,
                 )
 
+        if self.proc_hdu and "IPA" in self.proc_hdu.header:
+            rotator_angle = round(self.proc_hdu.header["IPA"], 2)
+        else:
+            rotator_angle = -999.0
+
+        headers = {
+            "rotator_angle": rotator_angle,
+            "fvc_centroid_method": self.centroid_method or "?",
+            "fvc_rms": self.fitrms,
+            "fvc_90_perc": self.perc_90,
+            "fvc_percent_reached": self.fvc_percent_reached,
+            "fvc_image_path": self.proc_image_path if self.proc_image_path else "",
+        }
+        headers.update(extra_headers)
+
         await configuration_copy.write_summary(
+            path=path,
             flavour="F",
-            headers={
-                "fvc_rms": self.fitrms,
-                "fvc_90_perc": self.perc_90,
-                "fvc_percent_reached": self.fvc_percent_reached,
-                "fvc_image_path": self.proc_image_path if self.proc_image_path else "",
-            },
+            headers=headers,
             overwrite=True,
         )
+
+        # Plot analysis of FVC loop.
+        if plot and self.proc_image_path:
+            if self.configuration.assignment_data.boresight is None:
+                self.log(
+                    "Configuration does not have boresight set. "
+                    "Cannot produce FVC plots.",
+                    level=logging.WARNING,
+                )
+            else:
+                self.log("Creating FVC plots", level=logging.DEBUG)
+
+                outpath = str(self.proc_image_path).replace(".fits", "_distances.pdf")
+
+                plot_fvc_distances(
+                    self.configuration,
+                    configuration_copy.assignment_data.fibre_table,
+                    path=outpath,
+                )
+
+
+async def reprocess_configuration(
+    configuration_id: int,
+    path: pathlib.Path | str | None = None,
+    centroid_method: str | None = None,
+    use_suffix: bool = True,
+):
+    """Reprocesses the FVC image from a configuration with a different centroid method.
+
+    Outputs a new ``confSummaryF`` file.
+
+    Parameters
+    ----------
+    configuration_id
+        The configuration ID for which to reprocess data. Must have an existing
+        ``confSummaryF`` file in ``$SDSSCORE_DIR``.
+    path
+        The path where to write the new ``confSummaryF`` file. If `None`, defaults
+        to ``$SDSSCORE_DIR``.
+    centroid_method
+        The centroid method to use, one of ``"nudge"``, ``"sep"``, ``"winpos"``,
+        or ``"simple"``.
+    use_suffix
+        If `True`, the new ``confSummaryF`` path file will have a suffix
+        including the centroid mode used.
+
+    Returns
+    -------
+    path
+        The path to the new ``confSummaryF`` file.
+
+    """
+
+    site = config["observatory"]
+    confSummaryF_path = Configuration._get_summary_file_path(
+        configuration_id,
+        site,
+        "F",
+    )
+
+    if not os.path.exists(confSummaryF_path):
+        raise FileNotFoundError(
+            f"Cannot find a confSummaryF file for {configuration_id}."
+        )
+
+    header, _ = read_confSummary(confSummaryF_path)
+
+    design = Design(
+        header["design_id"],
+        epoch=header["epoch"],
+        scale=header["focal_scale"],
+    )
+
+    fps = FPS.get_instance()
+    assert not fps.can, "This function cannot be called on a running FPS instance."
+
+    configuration = design.configuration
+    configuration.configuration_id = configuration_id
+
+    fvc = FVC(site)
+
+    proc_fimg = header["fvc_image_path"]
+    fimg = proc_fimg.replace("proc-", "")
+
+    posangles = fits.getdata(proc_fimg, "POSANGLES")
+    fiber_data = pandas.DataFrame(fits.getdata(proc_fimg, "FIBERDATA"))
+
+    positioner_coords = {}
+    for row in posangles:
+        positioner_coords[row["positionerID"]] = (row["alphaReport"], row["betaReport"])
+
+    fvc.process_fvc_image(
+        fimg,
+        positioner_coords,
+        configuration=configuration,
+        fibre_data=fiber_data,
+        centroid_method=centroid_method,
+        plot=False,
+    )
+
+    if path is None:
+        path = confSummaryF_path
+
+    path = str(path)
+
+    if use_suffix:
+        path = path.replace(".par", f"_{fvc.centroid_method}.par")
+
+    await fvc.write_summary_F(
+        path=path,
+        plot=False,
+        extra_headers={
+            "MJD": header["MJD"],
+            "obstime": header["obstime"],
+            "temperature": header["temperature"],
+        },
+    )
