@@ -10,6 +10,7 @@ from __future__ import annotations
 
 import logging
 import os
+import pathlib
 import warnings
 from copy import deepcopy
 from time import time
@@ -279,9 +280,10 @@ class BaseConfiguration:
             elif level == logging.ERROR:
                 self.command.error(msg)
 
-    def _initialise_grid(self):
+    def _initialise_grid(self, collision_buffer: float | None = None):
+        """Create a robot grid."""
 
-        self.robot_grid = get_robot_grid(self.fps)
+        self.robot_grid = get_robot_grid(self.fps, collision_buffer=collision_buffer)
 
         return self.robot_grid
 
@@ -311,6 +313,7 @@ class BaseConfiguration:
 
     async def get_paths(
         self,
+        collision_buffer: float | None = None,
         decollide: bool = True,
         simple_decollision: bool = False,
         resolve_deadlocks: bool = True,
@@ -324,6 +327,9 @@ class BaseConfiguration:
 
         Parameters
         ----------
+        collision_buffer
+            The collision buffer to use when generating paths. If `None`, defaults
+            to the configuration file value.
         decollide
             Runs the decollision routine.
         simple_decollision
@@ -346,7 +352,7 @@ class BaseConfiguration:
         assert isinstance(self, BaseConfiguration)
 
         # Just to be sure, reinitialise the grid.
-        self.robot_grid = self._initialise_grid()
+        self.robot_grid = self._initialise_grid(collision_buffer=collision_buffer)
 
         ftable = self.assignment_data.fibre_table
         alpha0, beta0 = config["kaiju"]["lattice_position"]
@@ -703,6 +709,7 @@ class BaseConfiguration:
     async def write_summary(
         self,
         flavour: str = "",
+        path: str | pathlib.Path | None = None,
         overwrite: bool = False,
         headers: dict = {},
         fibre_table: pandas.DataFrame | None = None,
@@ -866,11 +873,12 @@ class BaseConfiguration:
         fibermap = Table(fibermap)
         fibermap.sort(["positionerId", "fiberType"])
 
-        path = self._get_summary_file_path(
-            self.configuration_id,
-            self.assignment_data.observatory,
-            flavour,
-        )
+        if path is None:
+            path = self._get_summary_file_path(
+                self.configuration_id,
+                self.assignment_data.observatory,
+                flavour,
+            )
 
         if os.path.exists(path):
             if overwrite:
@@ -882,7 +890,7 @@ class BaseConfiguration:
         os.makedirs(os.path.dirname(path), exist_ok=True)
 
         write_ndarray_to_yanny(
-            path,
+            str(path),
             [fibermap],
             structnames=["FIBERMAP"],
             hdr=header,
@@ -929,32 +937,32 @@ class DitheredConfiguration(BaseConfiguration):
 
     def __init__(
         self,
-        configuration: BaseConfiguration,
+        parent_configuration: BaseConfiguration,
         radius: float,
         epoch: float | None = None,
     ):
 
-        assert configuration.design
+        self.parent_configuration = parent_configuration
+        assert self.parent_configuration.design
 
-        super().__init__(scale=configuration.scale)
+        super().__init__(scale=parent_configuration.scale)
 
-        self.parent_configuration: BaseConfiguration = configuration
         self.is_dither = True
 
-        self.design = configuration.design
+        self.design = self.parent_configuration.design
         self.design_id = self.design.design_id
 
         self.assignment_data = AssignmentData(
             self,
-            epoch=epoch,
-            computer_coordinates=False,
+            epoch=epoch or self.parent_configuration.epoch,
+            compute_coordinates=False,
             scale=self.scale,
         )
         self.assignment_data.fibre_table = (
             self.parent_configuration.assignment_data.fibre_table.copy()
         )
 
-        self.assignment_data.site.set_time(epoch)
+        self.assignment_data.site.set_time(self.parent_configuration.epoch)
 
         icrs_bore = ICRS([[self.design.field.racen, self.design.field.deccen]])
         self.assignment_data.boresight = Observed(
@@ -963,10 +971,6 @@ class DitheredConfiguration(BaseConfiguration):
             wavelength=INST_TO_WAVE["GFA"],
         )
 
-        assert self.assignment_data.site.time
-
-        self.epoch = self.assignment_data.site.time.jd
-
         self.radius = radius
 
         self.extra_summary_data = {
@@ -1034,9 +1038,10 @@ class DitheredConfiguration(BaseConfiguration):
         ftable.loc[:, "valid"] = 1
         self.assignment_data.validate()
 
-    async def get_paths(self):
+    async def get_paths(self, collision_buffer: float | None = None):
+        """Get trajectory paths."""
 
-        self.robot_grid = self._initialise_grid()
+        self.robot_grid = self._initialise_grid(collision_buffer=collision_buffer)
 
         await self.fps.update_position()
         positions = self.fps.get_positions_dict()
@@ -1675,13 +1680,13 @@ class AssignmentData(BaseAssignmentData):
         self,
         configuration: Configuration | DitheredConfiguration,
         epoch: float | None = None,
-        computer_coordinates: bool = True,
+        compute_coordinates: bool = True,
         scale: float | None = None,
     ):
 
         super().__init__(configuration, scale=scale)
 
-        if computer_coordinates:
+        if compute_coordinates:
             self.compute_coordinates(epoch)
 
     def compute_coordinates(self, jd: Optional[float] = None):
