@@ -16,10 +16,17 @@ import peewee
 from astropy import table
 from astropy.io import fits
 
-from sdssdb.peewee.sdss5db import targetdb
+from sdssdb.peewee.sdss5db import opsdb, targetdb
+
+from jaeger import config
 
 
-__all__ = ["load_holes", "load_fields"]
+__all__ = [
+    "load_holes",
+    "load_fields",
+    "get_designid_from_queue",
+    "match_assignment_hash",
+]
 
 
 def check_database(f):
@@ -32,10 +39,92 @@ def check_database(f):
     return wrapper
 
 
+@check_database
+def get_designid_from_queue(
+    pop: bool = True,
+    epoch_delay: bool = False,
+) -> tuple[int | None, float | None]:
+    """Pops a design from the queue.
+
+    Parameters
+    ----------
+    pop
+        If `True`, pops the next design from the queue. Otherwise just returns the
+        design ID.
+    epoch_delay
+        If `True`, calculates the delta epoch that optimises observing all the
+        queued designs with the same hash.
+
+    Returns
+    -------
+    result
+        A tuple with the design ID as the first element and the delta epoch, in
+        seconds as the second one. If ``epoch_delay=False``, the delta epoch is
+        `None`.
+
+
+    """
+
+    if pop:
+        design = opsdb.Queue.pop()
+    else:
+        design = (
+            opsdb.Queue.select()
+            .where(opsdb.Queue.position > 0)
+            .order_by(opsdb.Queue.position)
+            .first()
+        )
+
+    if design is None:
+        return (None, None)
+
+    if epoch_delay is False:
+        return (design.design_id, None)
+
+    # Get a list of all the entries in the queue that have the
+    # same hash as the first one.
+    hash = targetdb.Design.get_by_id(design.design_id).assignment_hash.hex
+
+    design_hashes = (
+        opsdb.Queue.select(opsdb.Queue.position)
+        .join(targetdb.Design, on=(targetdb.Design.design_id == opsdb.Queue.design_id))
+        .where(targetdb.Design.assignment_hash == hash)
+        .where(opsdb.Queue.position >= (1 if pop is False else -1))
+        .order_by(opsdb.Queue.position)
+        .tuples()
+    )
+
+    # This should not happen, but if the first entry does not have position 1,
+    # just leave because something weird happened. Note that the position can be -1
+    # if we have pop the design.
+    positions = list(zip(*design_hashes))[0]
+    if positions[0] > 1:
+        return (design.design_id, 0.0)
+
+    # Count how many consecutive positions there are.
+    n_designs: int = 1
+    if len(positions) > 1:
+        positions = [p if p >= 0 else p + 1 for p in positions]
+        for idx in range(1, len(positions)):
+            if positions[idx] == positions[idx - 1] + 1:
+                n_designs += 1
+            else:
+                break
+
+    # Cap to the number of designs after which we'll force a reconfiguration.
+    max_designs_epoch: int = config["configuration"]["max_designs_epoch"]
+    if n_designs > max_designs_epoch:
+        n_designs = max_designs_epoch
+
+    return (design.design_id, n_designs / 2 * 900.0)
+
+
 @check_database
 def load_holes(observatory: str):
     """Loads a list holes to ``targetdb.hole``."""
 
+    targetdb.database.become_admin()
+
     observatory_pk = targetdb.Observatory.get(label=observatory).pk
 
     row_start = 13
@@ -67,7 +156,12 @@ def load_holes(observatory: str):
 
 
 @check_database
-def load_fields(plan: str, files: list[str] = None, pattern: str = None):
+def load_fields(
+    plan: str,
+    files: list[str] | None = None,
+    pattern: str | None = None,
+    sequential_field: bool = False,
+):
     """Loads a series of field.
 
     Parameters
@@ -79,8 +173,14 @@ def load_fields(plan: str, files: list[str] = None, pattern: str = None):
     pattern
         Alternative to ``files``, a pattern to be used with ``glob``
         to retrieve a list of files to load.
+    sequential_field
+        If `False`, uses the filename to determine the field ID. Otherwise
+        sequentially increments the ``field_id`` field starting with the current
+        maximum value.
     """
 
+    targetdb.database.become_admin()
+
     if files is None and pattern:
         files = list(glob(pattern))
 
@@ -111,11 +211,18 @@ def load_fields(plan: str, files: list[str] = None, pattern: str = None):
         hdul = fits.open(file_)
 
         # Create field
-        field_id = int(file_.split("-")[-1][:-5])
+        if sequential_field is False:
+            field_id = int(file_.split("-")[-1][:-5])
+        else:
+            field_id = targetdb.Field.select(
+                peewee.fn.max(targetdb.Field.field_id)
+            ).scalar()
+            field_id += 1
+
         observatory = hdul[0].header["OBS"].upper()
-        racen = hdul[0].header["RACEN"]
-        deccen = hdul[0].header["DECCEN"]
-        PA = hdul[0].header["PA"]
+        racen = float(hdul[0].header["RACEN"])
+        deccen = float(hdul[0].header["DECCEN"])
+        PA = float(hdul[0].header["PA"])
         field_cadence = hdul[0].header["FCADENCE"]
         nexp = hdul[0].header["NEXP"]
 
@@ -184,3 +291,15 @@ def load_fields(plan: str, files: list[str] = None, pattern: str = None):
             ]
 
         targetdb.Assignment.insert_many(insert_data).execute(targetdb.database)
+
+
+def match_assignment_hash(design_id1: int, design_id2: int):
+    """Checks if the assignment hashes of two designs match."""
+
+    if design_id1 == design_id2:
+        return True
+
+    design1 = targetdb.Design.get_by_id(design_id1)
+    design2 = targetdb.Design.get_by_id(design_id2)
+
+    return design1.assignment_hash.hex == design2.assignment_hash.hex
